@@ -14,8 +14,10 @@ import sys
 sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
 
 import py
-from jinja import Environment
-from jinja.parser import Parser
+from jinja2 import Environment
+from jinja2.loaders import BaseLoader
+from jinja2.exceptions import TemplateNotFound
+
 
 try:
     # This code adds support for coverage.py (see
@@ -25,15 +27,15 @@ try:
 
     import coverage, atexit
 
-    IGNORED_MODULES = ['jinja._speedups', 'jinja.defaults',
-                       'jinja.translators']
+    IGNORED_MODULES = ['jinja2._speedups', 'jinja2.defaults',
+                       'jinja2.translators']
 
     def report_coverage():
         coverage.stop()
         module_list = [
             mod for name, mod in sys.modules.copy().iteritems() if
             getattr(mod, '__file__', None) and
-            name.startswith('jinja.') and
+            name.startswith('jinja2.') and
             name not in IGNORED_MODULES
         ]
         module_list.sort()
@@ -52,43 +54,32 @@ except ImportError:
     coverage = None
 
 
-class GlobalLoader(object):
+class GlobalLoader(BaseLoader):
+    scope = globals()
 
-    def __init__(self, scope):
-        self.scope = scope
-
-    def get_source(self, environment, name, parent, scope=None):
-        return self.scope[name.upper() + 'TEMPLATE']
-
-    def parse(self, environment, name, parent, scope=None):
-        return Parser(environment, self.get_source(environment, name,
-                      parent, scope), name).parse()
-
-    def load(self, environment, name, translator, scope=None):
-        return translator.process(environment, self.parse(environment,
-                                  name, None, scope))
+    def get_source(self, environment, name):
+        try:
+            return self.scope[name.upper() + 'TEMPLATE'], None, None
+        except KeyError:
+            raise TemplateNotFound(name)
 
 
-loader = GlobalLoader(globals())
-simple_env = Environment(trim_blocks=True, friendly_traceback=False, loader=loader)
+loader = GlobalLoader()
+simple_env = Environment(trim_blocks=True, loader=loader, cache_size=0)
 
 
-class MemcacheClient(object):
-    """
-    Helper for the loader test.
-    """
+class Directory(py.test.collect.Directory):
 
-    def __init__(self, hosts):
-        self.cache = {}
+    def run(self):
+        rv = super(Directory, self).run()
+        if self.fspath.basename == 'tests':
+            rv.append('doctests')
+        return rv
 
-    def get(self, name):
-        return self.cache.get(name)
-
-    def set(self, name, data, time):
-        self.cache[name] = data
-
-sys.modules['memcache'] = memcache = type(sys)('memcache')
-memcache.Client = MemcacheClient
+    def join(self, name):
+        if name == 'doctests':
+            return JinjaDocTestModule(name, parent=self)
+        return super(Directory, self).join(name)
 
 
 class Module(py.test.collect.Module):
@@ -118,15 +109,41 @@ class JinjaTestFunction(py.test.collect.Function):
 
 class JinjaDocTest(py.test.collect.Item):
 
+    def __init__(self, *args, **kwargs):
+        realmod = kwargs.pop('realmod', False)
+        super(JinjaDocTest, self).__init__(*args, **kwargs)
+        self.realmod = realmod
+
     def run(self):
-        mod = py.std.types.ModuleType(self.name)
-        mod.__doc__ = self.obj
+        if self.realmod:
+            mod = __import__(self.name, None, None, [''])
+        else:
+            mod = py.std.types.ModuleType(self.name)
+            mod.__doc__ = self.obj
+            mod.env = self.parent.env
+            mod.MODULE = self.parent.obj
         self.execute(mod)
 
     def execute(self, mod):
-        mod.env = self.parent.env
-        mod.MODULE = self.parent.obj
         failed, tot = py.compat.doctest.testmod(mod, verbose=True)
         if failed:
             py.test.fail('doctest %s: %s failed out of %s' % (
                          self.fspath, failed, tot))
+
+
+class JinjaDocTestModule(py.test.collect.Module):
+
+    def __init__(self, *args, **kwargs):
+        super(JinjaDocTestModule, self).__init__(*args, **kwargs)
+        self.doctest_modules = [
+            'jinja2.environment', 'jinja2.compiler', 'jinja2.parser',
+            'jinja2.lexer', 'jinja2.ext', 'jinja2.sandbox',
+            'jinja2.filters', 'jinja2.tests', 'jinja2.utils',
+            'jinja2.runtime'
+        ]
+
+    def run(self):
+        return self.doctest_modules
+
+    def join(self, name):
+        return JinjaDocTest(name, parent=self, realmod=True)
