@@ -6,8 +6,8 @@
     :copyright: 2007 by Armin Ronacher.
     :license: BSD, see LICENSE for more details.
 """
-from jinja import Environment, DictLoader
-from jinja.exceptions import TemplateSyntaxError
+from jinja2 import Environment, DictLoader
+from jinja2.exceptions import TemplateSyntaxError
 
 LAYOUTTEMPLATE = '''\
 |{% block block1 %}block 1 from layout{% endblock %}
@@ -36,15 +36,6 @@ LEVEL4TEMPLATE = '''\
 {% block block3 %}block 3 from level4{% endblock %}
 '''
 
-BROKENTEMPLATE = '''\
-{% extends "layout" %}
-{% if false %}
-  {% block block1 %}
-    this is broken
-  {% endblock %}
-{% endif %}
-'''
-
 WORKINGTEMPLATE = '''\
 {% extends "layout" %}
 {% block block1 %}
@@ -100,14 +91,43 @@ def test_super():
     assert tmpl.render() == '--INTRO--|BEFORE|[(INNER)]|AFTER'
 
 
-def test_broken(env):
-    try:
-        tmpl = env.get_template('broken')
-    except TemplateSyntaxError:
-        pass
-    else:
-        raise RuntimeError('no syntax error occured')
-
-
 def test_working(env):
     tmpl = env.get_template('working')
+
+
+def test_reuse_blocks(env):
+    tmpl = env.from_string('{{ self.foo() }}|{% block foo %}42{% endblock %}|{{ self.foo() }}')
+    assert tmpl.render() == '42|42|42'
+
+
+def test_preserve_blocks():
+    env = Environment(loader=DictLoader({
+        'a': '{% if false %}{% block x %}A{% endblock %}{% endif %}{{ self.x() }}',
+        'b': '{% extends "a" %}{% block x %}B{{ super() }}{% endblock %}'
+    }))
+    tmpl = env.get_template('b')
+    assert tmpl.render() == 'BA'
+
+
+def test_dynamic_inheritance():
+    env = Environment(loader=DictLoader({
+        'master1': 'MASTER1{% block x %}{% endblock %}',
+        'master2': 'MASTER2{% block x %}{% endblock %}',
+        'child': '{% extends master %}{% block x %}CHILD{% endblock %}'
+    }))
+    tmpl = env.get_template('child')
+    for m in range(1, 3):
+        assert tmpl.render(master='master%d' % m) == 'MASTER%dCHILD' % m
+
+
+def test_multi_inheritance():
+    env = Environment(loader=DictLoader({
+        'master1': 'MASTER1{% block x %}{% endblock %}',
+        'master2': 'MASTER2{% block x %}{% endblock %}',
+        'child': '''{% if master %}{% extends master %}{% else %}{% extends
+                    'master1' %}{% endif %}{% block x %}CHILD{% endblock %}'''
+    }))
+    tmpl = env.get_template('child')
+    assert tmpl.render(master='master2') == 'MASTER2CHILD'
+    assert tmpl.render(master='master1') == 'MASTER1CHILD'
+    assert tmpl.render() == 'MASTER1CHILD'
